## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import Popup as PyPopup

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer

class CustomFunctions:

    def __init__(self):
		
            self.Distance = [[0] * 11 for i in range(11)]
            for iX in range(11):
                for iY in range(11):
                    if iX in [0, 1] and iY in [0, 1]:
                        self.Distance[iX][iY] = 1
                    else:
                        self.Distance[iX][iY] = int((iX**2 + iY**2)**0.5 + 0.75)

            self.CultureDistance = []
            for i in range(16):
                self.CultureDistance.append(0)
            for i in range(16):
                if i in [0, 1]:
                    self.CultureDistance[i] = 1
                else:
                    self.CultureDistance[i] = (0.75)**(i-1)

    def ChangeOldPlotCulture(self, iPlayer, pCity, Amount):

            if Amount == 0: #If this is true, none of this matters.
                return
#           This function mimics the way plot culture is assigned.  Works for a radius of up to 10.
            CityX = pCity.getX()
            CityY = pCity.getY()
            TargetCultureLevel = 0
            CurrentCulture = pCity.getCulture(iPlayer)
#           First, determine what the current culture level is for the player this culture is being assigned to.
            for i in range(gc.getNumCultureLevelInfos()):
                if CurrentCulture >= 50 * 5**TargetCultureLevel:
                    TargetCultureLevel += 1
            TargetCultureLevel += 1
#           Cycle through every plot within a 21x21 square centered on the city and assign plot culture.
            for iiX in range(21):
                for iiY in range(21):
                    iX = iiX-10 + CityX
                    iY = iiY-10 + CityY
                    FromX = abs(iiX - 10)
                    FromY = abs(iiY - 10)
                    if CyMap().isPlot(iX, iY):
                        if TargetCultureLevel - self.Distance[FromX][FromY] >= 0:
                            if Amount > 0:
                                CultureChange = ((TargetCultureLevel - self.Distance[FromX][FromY]) * 20 + Amount + 1)
                                CyMap().plot(iX, iY).changeCulture(iPlayer, CultureChange)
                            else:
                                CultureChange = ((TargetCultureLevel - self.Distance[FromX][FromY]) * 20 + -Amount + 1)
                                CyMap().plot(iX, iY).changeCulture(iPlayer, -CultureChange)

    def ChangeNewPlotCulture(self, iPlayer, pCity, Amount):

            if Amount == 0: #If this is true, none of this matters.
                return
#           This function assigns culture in a new way.  Works for a radius of up to 10.
            CityX = pCity.getX()
            CityY = pCity.getY()
            TargetCultureLevel = 0
            CurrentCulture = pCity.getCulture(iPlayer)
#           First, determine what the current culture level is for the player this culture is being assigned to.
            for i in range(gc.getNumCultureLevelInfos()):
                if CurrentCulture >= 50 * 5**TargetCultureLevel:
                    TargetCultureLevel += 1
            TargetCultureLevel += 1
#           Cycle through every plot within a 21x21 square centered on the city and assign plot culture.
            for iiX in range(21):
                for iiY in range(21):
                    iX = iiX-10 + CityX
                    iY = iiY-10 + CityY
                    FromX = abs(iiX - 10)
                    FromY = abs(iiY - 10)
                    if CyMap().isPlot(iX, iY):
                        if TargetCultureLevel - self.Distance[FromX][FromY] >= 0:
                            if Amount > 0:
                                CultureChange = int(Amount * self.CultureDistance[self.Distance[FromX][FromY]] + 1)
                                CyMap().plot(iX, iY).changeCulture(iPlayer, CultureChange)
                            else:
                                CultureChange = int(-Amount * self.CultureDistance[self.Distance[FromX][FromY]] + 1)
                                CyMap().plot(iX, iY).changeCulture(iPlayer, -CultureChange)

    def Inquisit(self, pCity):
        
            szReligionList = [ ]
            StateBelief = gc.getPlayer(pCity.getOwner()).getStateReligion()
            for i in range(gc.getNumReligionInfos()):
                if (not StateBelief == i and pCity.isHasReligion(i) and not pCity.isHolyCityByType(i)):
                    szReligionList.append(i)
            if len(szReligionList) != 0:
                iTarget = szReligionList[CyRandom().get(len(szReligionList), "Get a random number for religion removal.")]
                pCity.setHasReligion(iTarget, False, True, True)
                for i in range(gc.getNumBuildingInfos()):
                    if gc.getBuildingInfo(i).getPrereqReligion() == iTarget:
                        pCity.setHasRealBuilding(i, False)
            CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'The Inquisition strikes again!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/Actions/Pillage.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)

    def GiveFoodBonus(self, pCity):
#               Check this city for bonuses 9-19 and 28 - 30 and appropriate buildings and determine food bonus
                FoodBonus = 0
                FoodBonuses = [ gc.getInfoTypeForString('BONUS_BANANA'), gc.getInfoTypeForString('BONUS_CLAM'), gc.getInfoTypeForString('BONUS_CORN'), gc.getInfoTypeForString('BONUS_COW'), gc.getInfoTypeForString('BONUS_CRAB'), gc.getInfoTypeForString('BONUS_DEER'), gc.getInfoTypeForString('BONUS_FISH'), gc.getInfoTypeForString('BONUS_PIG'), gc.getInfoTypeForString('BONUS_RICE'), gc.getInfoTypeForString('BONUS_SHEEP'), gc.getInfoTypeForString('BONUS_WHEAT') ]
                GranaryBonuses = [ gc.getInfoTypeForString('BONUS_CORN'), gc.getInfoTypeForString('BONUS_RICE'), gc.getInfoTypeForString('BONUS_WHEAT') ]
                GrocerBonuses = [ gc.getInfoTypeForString('BONUS_BANANA'), gc.getInfoTypeForString('BONUS_SUGAR'), gc.getInfoTypeForString('BONUS_SPICES'), gc.getInfoTypeForString('BONUS_WINE'), gc.getInfoTypeForString('BONUS_SALT') ]
                SupermarketBonuses = [ gc.getInfoTypeForString('BONUS_COW'), gc.getInfoTypeForString('BONUS_DEER'), gc.getInfoTypeForString('BONUS_PIG'), gc.getInfoTypeForString('BONUS_SHEEP') ]
                HarborBonuses = [ gc.getInfoTypeForString('BONUS_CLAM'), gc.getInfoTypeForString('BONUS_CRAB'), gc.getInfoTypeForString('BONUS_FISH') ]
#                FoodBonusBuildings = [ gc.getInfoTypeForString('BUILDING_GRANARY'), gc.getInfoTypeForString('BUILDING_GROCER'), gc.getInfoTypeForString('BUILDING_SUPERMARKET'), gc.getInfoTypeForString('BUILDING_GRANARY')
#                FoodBonuses = [ BonusTypes.BONUS_BANANA, BonusTypes.BONUS_CLAM, BonusTypes.BONUS_CORN, BonusTypes.BONUS_COW, BonusTypes.BONUS_CRAB, BonusTypes.BONUS_DEER, BonusTypes.BONUS_FISH, BonusTypes.BONUS_PIG, BonusTypes.BONUS_RICE, BonusTypes.BONUS_SHEEP, BonusTypes.BONUS_WHEAT ]
#                GranaryBonuses = [ BonusTypes.BONUS_CORN, BonusTypes.BONUS_RICE, BonusTypes.BONUS_WHEAT ]
#                GrocerBonuses = [ BonusTypes.BONUS_BANANA, BonusTypes.BONUS_SUGAR, BonusTypes.BONUS_SPICES, BonusTypes.BONUS_WINE ]
 #               SupermarketBonuses = [ BonusTypes.BONUS_COW, BonusTypes.BONUS_DEER, BonusTypes.BONUS_PIG, BonusTypes.BONUS_SHEEP ]
#                HarborBonuses = [ BonusTypes.BONUS_CLAM, BonusTypes.BONUS_CRAB, BonusTypes.BONUS_FISH ]
                if (not pCity.isDisorder()):
                    for i in range(gc.getNumBonusInfos()):
                        if pCity.hasBonus(i):
                            if i in FoodBonuses:
                                FoodBonus += 0.05
                            if pCity.hasBuilding(gc.getInfoTypeForString('BUILDING_GRANARY')):
                                if i in GranaryBonuses:
                                    FoodBonus += 0.05
                            if pCity.hasBuilding(gc.getInfoTypeForString('BUILDING_GROCER')):
                                if i in GrocerBonuses:
                                    FoodBonus += 0.05
                            if pCity.hasBuilding(gc.getInfoTypeForString('BUILDING_SUPERMARKET')):
                                if i in SupermarketBonuses:
                                    FoodBonus += 0.05
                                if i == gc.getInfoTypeForString('BONUS_SALT'):
                                    FoodBonus -= 0.05
                            if pCity.hasBuilding(gc.getInfoTypeForString('BUILDING_HARBOR')):
                                if i in HarborBonuses:
                                    FoodBonus += 0.05
                    FoodBonus += 0.05 * pCity.getFreeSpecialistCount(gc.getInfoTypeForString('SPECIALIST_HYDROPONICS_FARM'))
                    FoodBonus = int(pCity.getYieldRate(YieldTypes.YIELD_FOOD) * FoodBonus)
                    if not pCity.isFoodProduction():
                        pCity.changeFood(FoodBonus)
                    else:
                        pCity.changeProduction(FoodBonus)

    def GiveProductionBonus(self, pCity):
#               Check this city for bonuses 0 to 8 (Alunimum, horses, coal, oil, ect)
                ProductionBonus = 0
                StrategicBonuses = [ gc.getInfoTypeForString('BONUS_ALUMINUM'), gc.getInfoTypeForString('BONUS_COAL'), gc.getInfoTypeForString('BONUS_COPPER'), gc.getInfoTypeForString('BONUS_HORSE'), gc.getInfoTypeForString('BONUS_IRON'), gc.getInfoTypeForString('BONUS_MARBLE'), gc.getInfoTypeForString('BONUS_OIL'), gc.getInfoTypeForString('BONUS_STONE'), gc.getInfoTypeForString('BONUS_URANIUM') ]
                if (not pCity.isDisorder()):
                    for i in range(gc.getNumBonusInfos()):
                        if i in StrategicBonuses and pCity.hasBonus(i):
                            ProductionBonus += 0.05
                    ProductionBonus = int(pCity.getBaseYieldRate(YieldTypes.YIELD_PRODUCTION) * ProductionBonus)
                    pCity.changeProduction(ProductionBonus)

    def AssignCustomCulture(self, pCity, iOwner):
#               Get number of players for next 2 functions
               
                NumPlayers = 0
		for i in range(gc.getMAX_CIV_PLAYERS()):
			if (gc.getPlayer(i).isAlive()):
				loopPlayer = PyPlayer(gc.getPlayer(i).getID())
				if (loopPlayer.isNone() or not loopPlayer.isAlive()):
					continue
				else:
					NumPlayers += 1

#               Perform Culture Assimilation - Disabled, 1% was far too high
#                for i in range(NumPlayers):
#                    if not i == iOwner:
#                        CultureAssimilation = int(-pCity.getCulture(i) * 0.01)
#                        pCity.changeCulture(i, CultureAssimilation, False)

                CultureRate = pCity.getCommerceRate(gc.getInfoTypeForString('COMMERCE_CULTURE'))
                if pCity.getCommerceRate(gc.getInfoTypeForString('COMMERCE_CULTURE')) == pCity.getBaseCommerceRate(gc.getInfoTypeForString('COMMERCE_CULTURE')):
                    CultureModifier = 1
                else:
                    CultureModifier = pCity.getCommerceRate(gc.getInfoTypeForString('COMMERCE_CULTURE')) / pCity.getBaseCommerceRate(gc.getInfoTypeForString('COMMERCE_CULTURE'))
                CultureChange = []
                for i in range(NumPlayers):
                    CultureChange.append(0)
#               Assign culture based on nationality
                for i in range(NumPlayers):
                    if not i == iOwner:
                        CultureNationality = int((pCity.plot().calculateCulturePercent(i) / 100.0 - (0.4 * (pCity.plot().calculateCulturePercent(i)/100.0)**2)) * CultureRate)
                        CultureChange[i] += CultureNationality
                    else:
                        CultureNationality = int( (0.4 + 0.6 * pCity.plot().calculateCulturePercent(i) / 100.0) * CultureRate + 1)
                        CultureChange[i] += CultureNationality

#		Assign culture based on trade routes
#               Culture is assigned based on the culture rate of the trade city, the nationality of the trade city, and the value of the trade city.
#               A mixed nationality trade city will spread its mixed nationality.
#               A culturally strong trade city will spread more culture per commerce than a poor one.
#               A economically strong trade city will spread more culture than an economically poor one.
                routes = pCity.getTradeRoutes()
                for i in range(routes):
                    if (pCity.getTradeCity(i).getName()):
                        tradecity = pCity.getTradeCity(i)
                        Nation = tradecity.getOwner()
                        iTradeValue = tradecity.calculateTradeProfit(pCity)
			TradeCityCultureRate = tradecity.getCommerceRate(gc.getInfoTypeForString('COMMERCE_CULTURE')) * iTradeValue * 2 / 100.0
                        for n in range(NumPlayers):
                            if not n == iOwner:
                                TradeCultureNationality = int(tradecity.plot().calculateCulturePercent(n) * TradeCityCultureRate / 100.0)
                                CultureChange[n] += TradeCultureNationality
                            else:
                                TradeCultureNationality = int(tradecity.plot().calculateCulturePercent(n) * TradeCityCultureRate / 200.0)
                                CultureChange[n] += TradeCultureNationality

#		Assign culture to the holy city owners' nations based on religions present.
		for i in range(gc.getNumReligionInfos()):
                    if pCity.isHasReligion(i):
			HolyCityCity = CyGame().getHolyCity(i)
			Nation = HolyCityCity.getOwner()
                        CultureAmount = 1
                        for n in range(gc.getNumBuildingInfos()):
                            if pCity.hasBuilding(n):
                                if gc.getBuildingInfo(n).getPrereqReligion() == i:
                                    CultureAmount += 1
                        CultureAmount = int(CultureAmount * CultureModifier)
                        CultureChange[Nation] += CultureAmount
                    
#               Remove the default culture rate
                pCity.changeCulture(iOwner, -CultureRate, False)
                self.ChangeOldPlotCulture(iOwner, pCity, -CultureRate)
#               Reassign culture based on the new model
                for i in range(NumPlayers):
                    pCity.changeCulture(i, CultureChange[i], False)
                    self.ChangeNewPlotCulture(i, pCity, CultureChange[i])
